use std::fs;

use rayon::iter::{ParallelBridge, ParallelIterator};

pub fn solve() {
    let path = "res/07/input";

    let contents = fs::read_to_string(path).expect("Something went wrong reading the file");

    let equations: Vec<(usize, Vec<usize>)> = contents
        .lines()
        .map(|line| {
            let (res_str, nums_str) = line.split_once(": ").unwrap();
            let res: usize = res_str.trim().parse().unwrap();
            let nums: Vec<usize> = nums_str
                .trim()
                .split(' ')
                .map(|num| num.trim().parse().unwrap())
                .collect();
            (res, nums)
        })
        .collect();

    let result: usize = equations
        .iter()
        .par_bridge()
        .filter(|(target_res, nums)| {
            // try all combinations of + and * between nums and check if res can be achieved
            let mut operator_bits = 0;
            loop {
                let mut res = nums[0];
                for i in 1..nums.len() {
                    if operator_bits & 1 << (i - 1) == 0 {
                        res += nums[i];
                    } else {
                        res *= nums[i];
                    }
                }
                if res == *target_res {
                    return true;
                }
                if operator_bits == 2usize.pow(nums.len() as u32) - 1 {
                    return false;
                }
                operator_bits += 1;
            }
        })
        .map(|(res, _)| res)
        .sum();

    println!("Result 1: {}", result);

    let result: usize = equations
        .iter()
        .par_bridge()
        .filter(|(target_res, nums)| {
            // try all combinations of + and * between nums and check if res can be achieved
            let mut operator_bits: usize = 0;
            loop {
                let mut concat_bits: usize = 0;
                loop {
                    let mut res = nums[0];
                    for i in 1..nums.len() {
                        if concat_bits & 1 << (i - 1) == 0 {
                            if operator_bits & 1 << (i - 1) == 0 {
                                res += nums[i];
                            } else {
                                res *= nums[i];
                            }
                        } else {
                            res = concat(res, nums[i]);
                        }
                    }
                    if res == *target_res {
                        return true;
                    }
                    if concat_bits == 2usize.pow(nums.len() as u32) - 1 {
                        break;
                    }
                    concat_bits += 1;
                }
                if operator_bits == 2usize.pow(nums.len() as u32) - 1 {
                    break;
                }
                operator_bits += 1;
            }
            false
        })
        .map(|(res, _)| res)
        .sum();

    println!("Result 2: {}", result);
}

fn concat(a: usize, b: usize) -> usize {
    a * 10usize.pow(b.ilog10() + 1) + b
}
